from os import environ

import jwt
from jwt.exceptions import ExpiredSignatureError

SECRET_KEY = environ.get("SECRET_KEY")
EXPIRE_TIME = environ.get("EXPIRE_TIME")
ALGORITHM = environ.get("ALGORITHM")


def create_token(data):
    """
    Create a token

    Args:
        data (dict): data to be encoded in the token

    Returns:
        str: token
    """
    return jwt.encode(data, key=SECRET_KEY, algorithm=ALGORITHM)


def verify_token(token):
    """
    Verify a token

    Args:
        token (str): token to be verified

    Returns:
        dict: decoded token
    """
    try:
        return jwt.decode(token, key=SECRET_KEY, algorithms=ALGORITHM, verify=True)
    except ExpiredSignatureError:
        return False



from datetime import datetime, timedelta
from os import environ

import jwt
from fastapi.security import OAuth2PasswordBearer
from passlib.context import CryptContext

SECRET_KEY = environ.get("SECRET_KEY")
ALGORITHM = "HS256"

pwd_context = CryptContext(schemes=["sha256_crypt"], deprecated="auto")
oauth2_scheme = OAuth2PasswordBearer(tokenUrl=environ.get("TOKEN_URL"))


def create_access_token(*, data: dict, expires_delta: timedelta = None):
    """
    Create an access token from a dictionary of data.
    Args:
        data (dict): Dictionary of data to encode in the token.
        expires_delta (timedelta, optional): Time delta for token expiration. Defaults to None.
    Returns:
        str: Access token.
    """
    to_encode = data.copy()
    if expires_delta:
        expire = datetime.utcnow() + expires_delta
    else:
        expire = datetime.utcnow() + timedelta(minutes=15)
    to_encode.update({"exp": expire})
    return jwt.encode(to_encode, SECRET_KEY, algorithm=ALGORITHM)



import setuptools

__version__ = "1.2.5"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fastapi_lazy",
    version=__version__,
    author="Yasser Tahiri",
    author_email="yasserth19@gmail.com",
    description="Utilities that you use in various projects made in FastAPI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yezz123/fastapi-lazy",
    packages=setuptools.find_packages(
        exclude=["tests", "tests.*", "*.tests", "*.tests.*"],
    ),
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Internet :: WWW/HTTP :: Session",
    ],
    python_requires=">=3.7",
    install_requires=[
        "fastapi",
        "databases",
        "motor",
        "pyjwt",
        "aioredis==2.0.0",
        "email-validator",
        "passlib",
    ],
)



from os import environ

from fastapi.exceptions import HTTPException
from fastapi.security import HTTPBearer
from jwt import decode
from pydantic import BaseModel
from starlette.requests import Request
from starlette.status import HTTP_403_FORBIDDEN

KEY = environ.get("KEY", "secret")

# Authenticate Model
class LazyUser(BaseModel):
    """
    A lazy user model.

    Args:
        BaseModel (pydantic.BaseModel): Base model class.
    """

    username: str
    password: str
    role: str


class LazyAuth(HTTPBearer):
    """
    A custom authentication class that uses the JWT token to authenticate the user.

    Args:
        HTTPBearer (class): The class that handles the authentication.
    """

    async def __call__(self, request: Request) -> LazyUser:
        """
        The authentication method.

        Args:
            request (Request): The request object.

        Raises:
            HTTPException: If the token is invalid.

        Returns:
            LazyUser: The user object.
        """
        cred = await super().__call__(request)
        try:
            user = decode(cred.credentials, key=KEY, algorithms="HS256")
        except Exception as exp:
            """
                If the token is invalid, raise an HTTPException.

                Raises:
                    HTTPException: If the token is invalid.
            """
            raise HTTPException(
                status_code=HTTP_403_FORBIDDEN, detail=str(exp),
            )
        return LazyUser(**user)


authentication_lazy = LazyAuth(scheme_name="Bearer", auto_error=False,)
